// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function sopi_plot2DProblem(problem, var1, var2)
   if problem.nvar ~= 2 then
      error('Can only plot 2D problems')
   end
   // Optimal values
   if argn(2) < 3 then
      xopt     = var1.opt(1)
      yopt     = var2.opt(2)
      vecVar   = %T
      x_id     = 1
      y_id     = 2
   else
      xopt     = var1.opt
      yopt     = var2.opt
      vecVar   = %F
      x_id = sopi_getVarIdxInProblem(problem, var1)
      y_id = sopi_getVarIdxInProblem(problem, var2)
   end
   // Domain
   c     = 50
   xLim  = [xopt - c, xopt + c]
   yLim  = [yopt - c, yopt + c]
   h     = 1
   x     = xLim(1):h:xLim(2)
   y     = yLim(1):h:yLim(2)
   // Constraints
   // Linear inequality constraints
   if ~isempty(problem.A) then
      for i = 1:size(problem.A,1)
         if and(problem.A(i,:)~=0)
            cxi   = problem.A(i,x_id)
            cyi   = problem.A(i,y_id)
            bi    = problem.b(i)
            sopi_plot2DLIC(cxi, cyi, bi, xLim, yLim)
         end
      end
   end
   // Bound constraints
   for i = 1:length(problem.lb)
      if i == x_id
         sopi_plotLB(problem.lb(i), 1, xLim, yLim)
         sopi_plotUB(problem.ub(i), 1, xLim, yLim)
      else
         sopi_plotLB(problem.lb(i), 2,  xLim, yLim)
         sopi_plotUB(problem.ub(i), 2,  xLim, yLim)
      end
   end
   // Linear equality constraints
   if ~isempty(problem.Aeq) then
      for i =1: size(problem.Aeq,1)
         if and(problem.Aeq(i,:)) ~=0
            cxi   = problem.Aeq(i, x_id)
            cyi   = problem.Aeq(i, y_id)
            bi    = problem.beq(i)
            sopi_plot2DLEC(cxi,cyi, bi, xLim,yLim)
         end
      end
   end
   // Nonlinear inequality constraints
   if ~isempty(problem.c) then
      deff('out = testIneq(z)',['out = %F';'if z< 0 then';'out = %T';'end'])
      for c = problem.c
         cfun        = sopi_toScilabFunction(c.lhs)
         [z, xo, yo] = sopi_meshEval(x,y,cfun,vecVar,testIneq)
         if ~isempty(xo)
            plot(xo,yo,'.', 'color', name2rgb('lightgray')/255)
         end
      end
   end
   // 2D contour of objective function
   objective   = sopi_toScilabFunction(problem.objective)
   z           = sopi_meshEval(x, y, objective, vecVar)
   nz          = 8;
   contour2d(x,y,z,nz)
   //
   plot(xopt, yopt, 'r.')
   xstring(xopt+0.1,yopt+0.1, 'xopt')
   name1 = []
   name2 = []
   if ~vecVar then
      if ~isempty(var1.name)
         name1 = var1.name
      end
      if ~isempty(var2.name)
         name2 = var2.name
      end
   end
   if isempty(name1) then
      name1 = 'x1'
   end
   if isempty(name2) then
      name2 = 'x2'
   end
   xlabel(name1)
   ylabel(name2)
   set(gca(),'data_bounds',[xLim(:),yLim(:)])
endfunction
//
function sopi_plot2DLEC(a, b, c, xLim, yLim)
   dark_brown = [128,48,0]/256.0
   if a == 0 then
      // y = c/b
      plot(xLim, c/b*[1,1],'color',dark_brown,'thickness',2)
   elseif b == 0
      // x = c/a
      plot(c/a*[1,1], yLim,'color',dark_brown,'thickness',2)
   else
      // y = c/b - a/b * x
      A = -a/b
      B = c/b
      x1 = (yLim(1)-B)/A
      if x1 < xLim(1)
         x1 = xLim(1)
         y1 = A*xLim(1) + B
      elseif x1 > xLim(2)
         x1 = xLim(2)
         y1 = A*xLim(2) + B
      else
         x1 = x1
         y1 = yLim(1)
      end
      x2 = (yLim(2)-B)/A
      if x2 < xLim(1)
         x2 = xLim(1)
         y2 = A*xLim(1) + B
      elseif x2 > xLim(2)
         x2 = xLim(2)
         y2 = A*xLim(2) + B
      else
         y2 = yLim(2)
      end
      plot([x1, x2], [y1,y2],'color',dark_brown,'thickness',2)
   end

endfunction
//
function sopi_plot2DLIC(a, b, c, xLim, yLim)
   // plot ax + by <= c
   A  = -a/b
   B  =  c/b
   // Intersection of the constraint with the bottom of the box yLim1
   x1 = (yLim(1) - B)/A
   if x1 < xLim(1) then
      xp = xLim(1)
      yp = A*xLim(1) + B
   elseif x1 > xLim(2) then
      xp = xLim(2)
      yp = A*xLim(2) + B
   else
      xp = x1
      yp = yLim(1)
   end
   // Intersection of the constraint with the top of the box yLim2
   x2 = (yLim(2) - B)/A
   if x2 < xLim(1) then
      xp = [xp;xLim(1)]
      yp = [yp; A*xLim(1) + B]
   elseif x2 > xLim(2) then
      xp = [xp; xLim(2)]
      yp = [yp; A*xLim(2) + B]
   else
      xp = [xp; x2]
      yp = [yp; yLim(2)]
   end
   //
   P = [xp,yp]
   Ps = sopi_sortRows(P,1)
   xp = Ps(:,1)
   yp = Ps(:,2)

   //
   left  = list()
   right = list()
   for i = 1:length(xLim)
      x2 = xLim(i)
      for j = 1:length(yLim)
         y2 = yLim(j)
         t = (xp(2) - xp(1)) * (y2 - yp(1)) - (x2 - xp(1))*(yp(2) - yp(1))
         if t >0 then
            left($+1) = list(x2,y2)
         elseif t<0
            right($+1) =  list(x2,y2)
         end
      end
   end
   // Vertices of the polytope are on the right or on the left depending on the
   // orientation of the constraint
   if b > 0 then
      points = left
   else
      points = right
   end
   // Adding the extra vertices to the polytope
   for p = points
      xp = [xp; p(1)]
      yp = [yp; p(2)]
   end
   //   plot(xp, yp,'r.')
   //   if length(xp)>3 then
   H = sopi_convexHull(xp,yp)
   xp = H(:,1)
   yp = H(:,2)
   //   end
   xfpoly(xp,yp,25)
endfunction

function [z, xo, yo] = sopi_meshEval(x, y, fun, vec, test)
   xo = []
   yo = []
   if argn(2) == 5 then
      doTest = %T
   else
      doTest = %F
   end
   z = zeros(length(x),length(y))
   for i = 1:length(x)
      for j = 1:length(y)
         if vec
            z(i,j) = fun([x(i), y(j)])
         else
            z(i,j) = fun(x(i), y(j))
         end
         if doTest
            if test(z(i,j))
               xo = [xo, x(i)]
               yo = [yo, y(j)]
            end
         end
      end
   end
endfunction
