// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function p = sopi_applyRules(rules, p, args)
   // Applies some rules to some arguments
   //
   // Calling Sequence
   //    p = sopi_applyRules(rules, p, args)
   //
   // Parameters
   //    rules : a set of rules to apply (list)
   //    p     : the output properties (sopiProp)
   //    args  : the arguments for the rules (list)
   //
   if isempty(p) then
      p = sopi_newProp()
   end
   IS = 'is'
   for r = rules
      select r.ruleType
      case 'all'
         satisfyPrecond = %T
         for ai = args
            aiIsOk = %F
            for s = r.sets
               aiInSets = %T
               for i = 1:size(s,1)
                  aiInSets = aiInSets & ai(IS + s(i))
               end
               aiIsOk = aiIsOk | aiInSets
            end
            satisfyPrecond = satisfyPrecond & aiIsOk
         end
         if satisfyPrecond
            p = p + r.out
            if typeof(r.postAction) == 'function'
               p = r.postAction(p,args)
            end
         end
      case 'incond'
         // Inconditional rule: it is always applied
         p = p + r.out
      case 'comp'
         applyOut1 = %T
         applyOut2 = %T
         for i = 1:2
            pi    = args(i)
            pii   = args(modulo(i,2) +1)
            for s = r('sets'+string(i))
               applyOut1 = applyOut1 & pi(IS + s)
               applyOut2 = applyOut2 & pii(IS + s)
            end
         end

         if applyOut1 | applyOut2
            p = p + r.out
//         else
         end
      end
   end
endfunction
