// Author : Pierre Vuillemin (2017)
// License : GNU GPL

function [xopt, fopt, info] = sopi_int_qp(problem, x0, options, optFields)
   // Internal quadratic problem solver of Sopi.
   
   
   if  or(problem.class(1) == [20,21]) & problem.class(2) == -1 then
      // Convex unconstrained case
      [xopt, fopt] = sopi_solveConvexUQP(problem.fun.H, problem.fun.c, problem.fun.q)
   end
   info = struct()
endfunction

// ============================================================================
// CONVEX QP
// ============================================================================

// sopi_solveConvexUQP ........................................................
// solves convex unconstrained quadratic problems.
function [xopt, fopt] = sopi_solveConvexUQP(H,c,r)
   if and(c == 0) then
      xopt = c
   else
      xopt = -H\c
   end
   fopt = xopt'*H*xopt + c'*xopt + r
endfunction

// sopi_solveECQP ..............................................................
// solves quadratic problems with linear equality constraints :
// min x'*H*x + c'*x + r s.t. A*x = b where H is symmetric
function [xopt, fopt] = sopi_solveECQP(H,c,r,A,b,method)
   // default method
   if argn(2)<6 then
      method = "full-space";
   end
   xopt        = [];
   fopt        = [];
   [m,n]       = size(A);
   select method
   case "full-space" // inversion of the KKT matrix
      kktMat      = [H,A';A,sparse([],[],[m,m])];
      rhs         = [-c;b];
      y           = kktMat\rhs;
      xopt        = y(1:n);
   case "elimination" // elimination of the equality constraints
      [Q,R]       = qr(A');
      R(m+1:$,:)  = [];
      R(:,m+1:$)  = []
      Q1          = Q(:,1:m);
      Q2          = Q(:,m+1:$);
      Yb          = Q1*(R'\b);
      Z           = Q2;
      y           = sopi_solveConvexUQP(Z'*H*Z,Z'*(c+H*Yb),0);//(Z'*H*Z)\(-Z'*(c+H*Yb));
      xopt        = Yb + Z*y;
   end
   fopt = xopt'*H*xopt + c'*xopt + r;
endfunction
