// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function [xopt, fopt, info] = sopi_karmarkar(problem, x0, options, optFields)
   // Attempt to solve a linear problem with the built-in method karmarkar.
   //
   // Calling Sequence
   //    [xopt, fopt, info] = sopi_karmarkar(problem, x0, options, optFields)
   //
   // Parameters
   //    problem  : the optimisation problem to solve (sopiPro)
   //    x0       : the initial point (array, optional)
   //    options  : possible options for the solver (structure)
   //    optFields: name of the possible options (list)
   //    xopt     : optimal solution (array)
   //    fopt     : optimal objective function (double)
   //    info     : informations about the optimisation process (structure)
   //
   
   // Optional elements
   sopi_tryToExtract(options, optFields)
   // Call of the solver
   [xopt, fopt, exitflag, iter, yopt] = karmarkar(problem.Aeq, problem.beq, problem.fun.c, x0, rtolf, gam, maxiter, outfun, problem.A, problem.b, problem.lb, problem.ub)
   fopt = fopt + problem.fun.q
   // Informations about the optimisation process
   info              = struct()
   info.solverInfo   = struct('exitflag',exitflag,'iter',iter,'yopt',yopt)
   select exitflag
   case 1
      vflag = 'The algorithm has converged'
   case 0
      vflag = sprintf('Maximum number of iterations (%d) reached',iter)
   case -1
      vflag = 'No feasible point found'
   case -2
      vflag = 'Problem is unbounded'
   case -3
      vflag = 'Search direction became zero'
   case -4
      vflag = 'Stopped on user''s request'
   end
   info.vflag   = vflag
   info.success = exitflag == 1
endfunction
