// Author : Pierre Vuillemin (2017)
// License : GNU GPL


function [xopt, fopt, info] = sopi_qld(problem, x0, options, optFields)
   // Attempt to solve a quadratic problem with the builtin method qld.
   //
   // Calling Sequence
   //    [xopt, fopt, info] = sopi_qld(problem, x0, options, optFields)
   //
   // Parameters
   //    problem  : the optimisation problem to solve (sopiPro)
   //    x0       : the initial point (array, optional)
   //    options  : possible options for the solver (structure)
   //    optFields: name of the possible options (list)
   //    xopt     : optimal solution (array)
   //    fopt     : optimal objective function (double)
   //    info     : informations about the optimisation process (structure)
   //
   sopi_tryToExtract(options, optFields)
   // Constraints matrices
   C                    = [problem.Aeq;problem.A]
   b                    = [problem.beq;problem.b]
   //
   if ~isempty(tol) then
      [xopt, lagr, exitflag]   = qld(2*problem.fun.H,problem.fun.c,C,b,problem.lb, problem.ub,size(problem.Aeq,1),tol)
   else
      [xopt, lagr, exitflag]   = qld(2*problem.fun.H,problem.fun.c,C,b,problem.lb, problem.ub,size(problem.Aeq,1))
   end
   // Problem solution
   fopt = xopt'*problem.fun.H*xopt + problem.fun.c'*xopt + problem.fun.q
   // Information about the optimisation process
   info.solverInfo = struct('exitFlag',exitflag,'lagr',lagr)
   select exitflag
   case 1
      vflag = 'Too many iterations needed'
   case 2
      vflag = 'Accuracy insufficient to satisfy convergence criterion'
   case 5
      vflag = 'Length of working array is too short'
   case 10
      vflag = 'The constraints are inconsistent'
   case 0
      vflag = 'The algorithm has converged'
   else
      error('Unknown flag')
   end
   info.vflag      = vflag
   info.success    = []
endfunction
