// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function problem = sopi_increaseNvar(problem, n)
   // Adapts the elements of an optimisation problem when the number of variables
   // increases
   //
   // Calling Sequence
   //    problem = sopi_increaseNvar(problem, n)
   //
   // Parameters
   //    problem  : optimisation problem (sopiPro)
   //    n        : number of additional variables (integer)
   //
   // Description
   //    When variables are added to an optimisation problem, the vector
   //    representing the constraints have to be updated to match the new size.
   //

   // TODO: update also other constraints
   if isempty(problem.ub) then
      ub = %inf*ones(n,1);
   else
      ub = [problem.ub;%inf*ones(n,1)];
   end
   if isempty(problem.lb) then
      lb = -%inf*ones(n,1);
   else
      lb = [problem.lb;-%inf*ones(n,1)];
   end
   problem.lb     = lb;
   problem.ub     = ub;
   problem.nvar   = problem.nvar + n
endfunction
