// Author : Pierre Vuillemin (2016)s
// License : GNU GPL

function sopi_end()
   // Clears all sopi-related variables.
   //
   // Calling Sequence
   //    sopi_end()
   //
   // Description
   //    Attempts to clean all sopi related variables and assign optimal values,
   //    if any.
   //
   // Examples
   //    // Initialisation of sopi
   //    sopi_begin();
   //    // Declaring a sopiVar
   //    x = sopi_var(1);
   //    disp(x);
   //    // Cleaning sopiVars
   //    sopi_end();
   //    disp(x);
   //
   // See Also
   //    sopi_begin
   //
   clearglobal sopiData
   varNames    = []
   varValues   = []
   vars        = sopi_getExistingSopiVar()
   if ~isempty(vars) then
      data     = list()
      for i = 1:length(vars)
         var      = vars(i)(1)
         varName  = vars(i)(2)
         varNames = [varNames, varName]
         if ~isempty(var.opt)
            data($+1)   = var.opt
            varValues   = [varValues, 'data(' + string(length(data))+')']
         else
//            varValues = [varValues, 'clear('''+varName+''')']
            varValues = [varValues, '[]'] // for compatibility with Scilab 6
         end
      end
      clear var
   end
   csts = sopi_getExistingSopiCst()
   if ~isempty(csts) then
      for i = 1:length(csts)
         cName       = csts(i)(2)
         varNames    = [varNames, cName]
//         varValues   = [varValues,'clear('''+cName+''')']
         // TODO: replace constraints by their maximum violation?
         varValues   = [varValues,'[]']
      end
   end
   probs = sopi_getExistingSopiPro()
   if ~isempty(probs) then
      for i = 1:length(probs)
         pName       = probs(i)(2)
         varNames    = [varNames, pName]
//         varValues   = [varValues,'clear('''+pName+''')']
         varValues   = [varValues,'[]']
      end
   end
   if ~isempty(varNames) then
      execstr(sopi_strToAssignTopVars(varNames, varValues))
   end
endfunction
