// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function sopi_print(printLvl,strList,varargin)
   // Prints a given string if its level is below the verbosity level of sopi.
   //
   // Calling Sequence
   //    sopi_print(printLvl, strList, varargin)
   //
   // Parameters
   //    printLvl : level of the message (int) or key (string)
   //    strList  : the message together with some tags (string or list)
   //    varargin : eventual data to be displayed
   //
   // Examples
   //    sopi_begin(verbosity = 1)
   //    sopi_print(2,'Should not be displayed')
   //    sopi_print(1,'Should be displayed')
   //    sopi_print(1,['tag1', 'Message'])
   //    sopi_print(1, 'Message with data: %d', 3)
   //    sopi_print(1,['tag1', 'Message with tag and data: %d'], 3)
   //
   [message, tagStr] = sopi_extractTags(strList);
   message           = ' ' + message;
   if sopi_verbosity()>= printLvl then
      if printLvl == -1 then
         tagStr = tagStr + '[WARNING]';
      end
      mprintf(tagStr + message, varargin(:));
   end
endfunction

function [message, tagStr] = sopi_extractTags(strList)
   // Divides the message from the tags given a string or matrix of strings.
   strList = ['sopi',strList];
   if  size(strList,2) > 1 then
      message     = strList($);
      tags        = strList(1:$-1);
      tagStr      = '[' + strcat(tags,'][') + ']';
   else
      message  = strList;
      tagStr   = [];
   end
endfunction
