// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function out = sopi_tabular(titl, header, entries)
   // Displays somewhat formated sopi_tabulars in the console.
   //
   // Calling Sequence
   //    tab = sopi_tabular(title, headers, entries)
   //
   // Parameters
   //    title    : title of the table (string)
   //    headers  : headers of each column (list)
   //    entries  : entries of the table (list)
   //    tab      : formated sopi_tabular (string)
   //
   // Description
   //    This routine enables to create formated tables to be printed in the
   //    console.
   //
   // Example
   //    titl     = 'This is a table';
   //    headers  = list('first column', 'second column', 'etc...');
   //    entries  = list(1, 'two', 3.0,4,5,6)
   //    tab      = sopi_tabular(titl, headers, entries)
   //    mprintf(tab)
   //
   ncol         = length(header)
   colLen       = zeros(ncol, 1)
   dispHeader   = %F
   for i = 1:ncol
      colLen(i) = length(header(i))
      if ~isempty(header(i))
         dispHeader = %T
      end
   end
   for i = 1:ncol:length(entries)
      for j = 1:ncol
         if typeof(entries(i + j - 1)) == 'constant' then
            entries(i + j - 1) = sprintf('%d', entries(i + j - 1))
         end
         colLen(j) = max(colLen(j), length(entries(i + j - 1)))
      end
   end
   totalLen    = sum(colLen) + ncol * 2 + ncol + 2
   line        = '+' + sopi_tabular_tiret(totalLen - 3) + '+\n'
   // Title
   titleLen    = length(titl)
   titleLeft   = floor((totalLen - titleLen)/2)
   titl        = sopi_tabular_blanks(titleLeft-1) + titl + sopi_tabular_blanks(totalLen - titleLeft-titleLen -2)
   out         = line + '|' + titl + '|\n'
   if dispHeader then
      out       = out + line
      out       = out + '| '
      for i = 1:length(header)
         if i > 1
            out = out + ' | '
         end
         if ~isempty(header(i))
            lentry = length(header(i))
            out    = out + header(i) + sopi_tabular_blanks(colLen(i)-lentry)
         else
            out = out + sopi_tabular_blanks(colLen(i))
         end
      end
      out    = out + ' | \n'
   end
   out       = out + line
   addLine   = %F
   for i = 1:ncol:length(entries)
      if part(entries(i),1) == '#' then
         out         = out + line
         entries(i)  = part(entries(i),2:length(entries(i)))
         addLine     = %T
      end
      out = out + '| '
      for j = 1:ncol
         lcol = length(entries(i + j - 1))
         if j > 1 then
            colStr = ' | ' + sopi_tabular_blanks(colLen(j) - lcol) + entries(i + j - 1)
         else
            colStr = entries(i + j - 1) + sopi_tabular_blanks(colLen(j) - lcol)
         end
         out = out + colStr
      end
      out = out + ' |\n'
      if addLine then
         out      = out + line
         addLine  = %F
      end
   end
   out = out + line
endfunction

function out = sopi_tabular_blanks(n)
   out = ''
   for i = 1:n
      out = out + ' '
   end
endfunction

function out = sopi_tabular_tiret(n)
   out = ''
   for i = 1:n
      out = out + '-'
   end
endfunction
